/***************************************************************************
                          display.h  -  description
                             -------------------
    begin                : Mon Jun 5 2000
    copyright            : (C) 2000 by Calvin Harrigan
    email                : charrig@viaccess.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// FILE:
// display.h
// FUNCTION:
// It's the header file for the display class.
// This class is part of the sunamp project.
// It is the wrapper for the matrix orbital display, but it has evolved to be the
// main class of the program.
// Copyright (c) 1999 Calvin Harrigan charrig@viaccess.net


#ifndef __DISPLAY_H__
#define __DISPLAY_H__

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "serial.h"
#include "xaudio.h"
#include "player.h"
#include "control_pipe.h"
#include "songlist.h"
#include "remote.h"

#define DRATE 400000
#define commandPrefix 0xFE
#define MAX_VOLUME 100
const int displayLength=20;
const int contStep=8;

typedef struct
{
        int currentPosition;            // start position in the string to scroll from
        char scrolled[100];             //string representation of the scrolled string
				char string[100];
        bool scrolling;                 // scrolling flag, if true line is scrolled.
        int length;                     // length of string being scrolled
}
ScrollType;


// array of eqband type structures used by the xaudio library
typedef struct
{
	int eqbands[10];
}
EQBands;

// Equalizer band labels
static const char * freq[]={" 60Hz","170Hz","310Hz","600Hz"," 1KHz"," 3KHz",
											" 6KHz","12KHz","14KHz","16KHz"};
class Display
{
public:      // some enumerations to cut down on confusion
	enum EVENT{XAUDIO,SERIAL,TIMEOUT,REMOTE};
	enum CONT{UP,DOWN};
	enum DISPMODE{PLAY,MENU,EQ,LISTS,CONTRAST};

	Display(){};
	Display(const char *);    // Main constructor
	~Display();               // Destructor
	int GetDescriptorSet(fd_set *)const; // returns the file descriptors for the pipe and serial port
	int GetFileDescriptors(int &, int &)const; // ditto
	int Event(EVENT);                          // event handling functions.  Events are detected by select()
	void ClearScreen()const;                   // clears display screen
	void BackLightOff(int)const;               // sets the backlight time out
	void SetContrast(CONT dir);                // sets display contrast
	void SetRemotePointer(Remote *);           // sets the pointer to the remote object
	int Quit()const{return quit;};             // returns the quit flag to the main function
	int GetKey()const;                         // return keys pressed on the display
	int Fail()const{return fFlag;};            // returns the fail flag
	int SetSongDir(char *);                    // Sets the main directory of the mp3s
private: 	
	//Variables
	int topGenre;        // genre currently at the top of the displayed list
	int genreIndex;      // genre currently pointed to on the display screen
	bool fFlag;          // fail flag
	linSerial *serPort;  // linSerial object pointer that controls the serial port
	XA_Player *player;   // Xaudio play object pointer
	SongList *songList;  // Pointer to songlist object to hold songs
	void * pControl;     // Pointer to xaudio control object
	int rxfd;            // rx audio pipe file descriptor
	int sfd;	           // serial port file descriptor
	int contrast;        // Current contrast setting for the player.
	FILE * fPort;        // stream pointer for the serial port
  fd_set set;          // file descriptor set of serial port and rxaudio pipe
	int quit;            // quit flag.  indicates a shutdown from the user interface
	int playerReady;     // Player process flag
	int playerState;     // Current Player process State
	int streamProgress;  // Percentage of song that has finished playing
	int streamMinutes, streamSeconds; // Elapsed minutes and seconds of the song
	int streamBitrate, streamFrequency, streamMode; // stream information returned from rxaudio
	DISPMODE dispMode;       // Current display mode
	int newXAudioData;       // flag that signals that new data is available from XAudio
	char *songDir;           // name of the working directory of the songs
	int	streamInputState;    // state of the input stream, open, close, etc.
	char playerSong[200];    // array to store the song that gets displayed
	ScrollType artistLine;   // structure used to scroll artist line
	ScrollType songLine;     // ditto for song line
	EQBands eqbands;         // structure to store the bands of the equalizer
	Remote * remotePointer;  // pointer to the remote control object
	int decoderPid;          // PID of the decoder process.
	XA_EqualizerInfo eqInfo; // equalizer settings storage type variable
	int pVolume;             // stores pcm volume settings
	int muteVolume;          // stores the current volume setting on mute

// Functions
	int ProcessKeyPress(int);     // processes the messages send from the "main" function
	void SetSong(char *);         // sets the currently selected song in the display structures
	int ProcessXaudioMessage(const XA_Message &);  // Process messages from the rxaudio process
// these functions are called depending on the current mode of the display
// object.  they are called with input from outside, 0 to initialize.
	int PlayScreen(int);
	int MenuScreen(int);
	int EqualizerScreen(int);
	int ListsScreen(int);
	int ContrastScreen(int);
	// int  SaySong();
};



#endif // __DISPLAY_H__
//============END OF FILE=====================
